/***************************************************************************
 *   Copyright (C) 2006-2007 by Vladimir Jary   *
 *   jaryv1@kmlinux.fjfi.cvut.cz   *
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 2 of the License, or     *
 *   (at your option) any later version.                                   *
 *                                                                         *
 *   This program is distributed in the hope that it will be useful,       *
 *   but WITHOUT ANY WARRANTY; without even the implied warranty of        *
 *   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the         *
 *   GNU General Public License for more details.                          *
 *                                                                         *
 *   You should have received a copy of the GNU General Public License     *
 *   along with this program; if not, write to the                         *
 *   Free Software Foundation, Inc.,                                       *
 *   59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.             *
 ***************************************************************************/


/*!
* \file mainform.ui.h
* \brief Main application dialog
*
* This form represents a wizard which will guide users through process
* of converting their videos into Theora format.
*/

/*!
* \class mainForm
* \brief Main dialog of ffmpeg2theora_fe application
*
* mainForm is a wizard which will guide users through process of converting their videos
* into Theora format.
*/

/*!
* \brief Initializes main from
*
* Called from form constructor it intializes member variables to their default values
* and installs validators to certain form fields.
*/
void mainForm::init()
{
	//install validators
	QIntValidator *validatorVQ = new QIntValidator(1,16000,this);
	QIntValidator *validatorAQ = new QIntValidator(32,500,this);
	QIntValidator *validator = new QIntValidator(this);
	validator->setBottom(0);
	editVideoRate->setValidator(validatorVQ);
	editAudioRate->setValidator(validatorAQ);
	editX->setValidator(validator);
	editY->setValidator(validator);
	editHz->setValidator(validator);
	editTop->setValidator(validator);
	editBottom->setValidator(validator);
	editLeft->setValidator(validator);
	editRight->setValidator(validator);
	//initialize member variables
	process = 0; //ffmpeg2theora process
	help = 0; //konqueror process for displaying man page
	parsed = false;
	editOutput->setText(QDir::homeDirPath()+"/");
	timeStart = 0;
	duration = 0;
	current = -1; //currently processed video
}

/*!
* \brief Displays form for inserting videos
*
* Displays form for inserting videos into queu. Establishes connection which
* is used to notify main form when new video inserted.
*/
void mainForm::showVideoForm()
{
	videoForm vf;
	connect(&vf,SIGNAL(inserted(QStringList)),this,SLOT(insertVideo(QStringList)));
	vf.setModal(true);
	vf.exec();
}

/*!
* \brief Inserts video into queue
*
* This slot is called from Insert Video form when user clicks Insert button Opened video together
* with tags is added into queue.
*/
void mainForm::insertVideo( QStringList tags )
{
	QListViewItem *item = new QListViewItem(viewSource);
	
	int i = 0;
	for ( QStringList::Iterator it = tags.begin(); it != tags.end(); ++it ) {
		
		item->setText(i,*it);
		i++;
	}
}

/*!
* \brief Selects output directory
*
* Displays dialog for choosing output directories
*/
void mainForm::selectOutput()
{
	QString directory = QFileDialog::getExistingDirectory(
			QDir::homeDirPath(),
			this,
			"get existing directory",
			tr("Choose an output directory"),
			TRUE );
	if(!directory.isEmpty())
	{
		editOutput->setText(directory);
	}
}


/*!
* \brief Returns list of arguments for ffmpeg2theora
*
* Parses user's input and prepares list of arguments for
* ffmpeg2theora binary.
* \param i currently processed item in task queue
* \return list of arguments for ffmpeg2theora
*/
QStringList mainForm::prepareArguments(QListViewItem *i)
{
    QStringList list;
    list << "ffmpeg2theora";

    list << i->text(0); //INPUT file

	//output file
    QFileInfo fi(i->text(0));
    QString ext = "." + fi.extension();
    QString outputFile = i->text(0).replace(ext,".ogg");
    list << "--output";

    if((radioDestination->isOn()) && (!editOutput->text().isEmpty()))
    {
	fi.setFile(outputFile);
	outputFile = editOutput->text()  + fi.fileName();
    }
    list << outputFile;
    
    
    if(radioPro->isChecked())
    {
	list << "--v2v-preset";
	list << "pro";
	
    }
    else if(radioPreview->isChecked())
    {
	list << "--v2v-preset";
	list << "preview";
    }
    else
    {
	//Audio
	if(!comboAudio->currentText().isEmpty())
	{
	    list << "--audioquality";
	    list << comboAudio->currentText();
	}
	
	if(!(editAudioRate->text().isEmpty()))
	{
	    list << "-A";
	    list << editAudioRate->text();
	}
	
	if(!(editHz->text().isEmpty()))
	{
	    list << "--samplerate";
	    list << editHz->text();
	}	
	
	if(!(editChannels->text().isEmpty()))
	{
	    list << "--channels";
	    list << editChannels->text();
	}
	
	if(checkSound->isOn())
	{
	    list << "--nosound";
	}
	
	//Video
	if(!comboVideo->currentText().isEmpty())
	{
	    list << "--videoquality";
	    list << comboVideo->currentText();
	}
	
	if(!(editVideoRate->text().isEmpty()))
	{
	    list << "-V";
	    list << editVideoRate->text();
	}
	
	//Crop
	if(!(editTop->text().isEmpty()))
	{
	    list << "--croptop";
	    list << editTop->text();
	}
	
	if(!(editBottom->text().isEmpty()))
	{
	    list << "--cropbottom";
	    list << editBottom->text();
	}
	
	if(!(editLeft->text().isEmpty()))
	{
	    list << "--cropleft";
	    list << editLeft->text();
	}
	
	if(!(editRight->text().isEmpty()))
	{
	    list << "--cropright";
	    list << editRight->text();
	}
	
	if(!(editX->text().isEmpty()))
	{
	    list << "--width";
	    list << editX->text();
	}
	
	if(!(editY->text().isEmpty()))
	{
	    list << "--height";
	    list << editY->text();
	}
	
	if(!(editAspect->text().isEmpty()))
	{
	    list << "--aspect";
	    list << editAspect->text();
	}
	
	if(!(editFPS->text().isEmpty()))
	{
	    list << "--inputfps";
	    list << editFPS->text();
	}
	
	if(!(editOutputFPS->text().isEmpty()))
	{
	    list << "--framerate";
	    list << editOutputFPS->text();
	}
	
	
	//Sharpness
	if(spinBoxSharpness->value()!=2)
	{
	    list << "-S";
	    list << spinBoxSharpness->text();
	}
	
	//Contrast
	if(spinBoxContrast->value()!=10)
	{
	    list << "-C";
	    list << QString("%1").arg(spinBoxContrast->value()/10.0,0,'f',1);
	}
	
	//Brightness
	if(spinBoxBrightness->value()!=0)
	{
	    list << "-B";
	    list << QString("%1").arg(spinBoxBrightness->value()/10.0,0,'f',1);
	}
	
	//Gamma
	if(spinBoxGamma->value()!=10)
	{
	    list << "-G";
	    list << QString("%1").arg(spinBoxGamma->value()/10.0,0,'f',1);	  
	}				
	
	//Saturation
	if(spinBoxSaturation->value()!=10)
	{
	    list << "-Z";
	    list << QString("%1").arg(spinBoxSaturation->value()/10.0,0,'f',1);
	}
	
	//Advanced options
	if(checkAdvanced->isOn()){
	    if(!editStart->text().isEmpty())
	    {
		list << "-s";
		list << editStart->text();
	    }
	    
	    if(!editEnd->text().isEmpty())
	    {
		list << "-e";
		list << editEnd->text();
	    }
	    if(spinBoxKeyframe->value()!=64)
	    {
		list << "-K";
		QString kf = QString::number(spinBoxKeyframe->value());
		list << kf;
	    }
	    
	    if(checkDeinterlace->isOn())
	    {
		list << "--deinterlace";
	    }
	    
	    if(checkOptimize->isOn())
	    {
		list << "--optimize";
	    }
	    
	    if(checkSync->isOn())
	    {
		list << "--sync";
	    }
	    
	    if(!editInFormat->text().isEmpty())
	    {
		list << "--format";
		list << editInFormat->text();
	    }
	    
	    if(!editAudioStream->text().isEmpty())
	    {
		list << "--audiostream";
		list << editAudioStream->text();
	    }
	    
	    if(!editNice->text().isEmpty())
	    {
		list << "--nice";
		list << editNice->text();
	    }
	    
	    if(!editV4L->text().isEmpty())
	    {
		list << "--v4l";
		list << editV4L->text();
	    }	
	}
    }

	//Tags
    if(!(i->text(1).isEmpty()))
    {
	list << "--artist";
	list << i->text(1);
    }

    if(!(i->text(2).isEmpty()))
    {
	list << "--title";
	list << i->text(2);
    }

    if(!(i->text(3).isEmpty()))
    {
	list << "--date";
	list << i->text(3);
    }

    if(!(i->text(4).isEmpty()))
    {
	list << "--location";
	list << i->text(4);
    }

    if(!(i->text(5).isEmpty()))
    {
	list << "--organization";
	list << i->text(5);
    }

    if(!(i->text(6).isEmpty()))
    {
	list << "--copyright";
	list << i->text(6);
    }

    if(!(i->text(7).isEmpty()))
    {
	list << "--licence";
	list << i->text(7);
    }

    return list;
}


/*!
*\brief Starts conversion process
*
* Starts conversion of current item in queue. At first, inforamation about current
* video is get from queue item and command-line arguments are parsed from form fields.
* Finally ffmpeg2theora process with these arguments is launched.
*/
void mainForm::startProcess()
{

    uint index = -1;

	//Get current item
    QListViewItem *i=0;
    QListViewItemIterator it( viewSource);
    while ( it.current() ) {
	index++;
	if(index==current)
	{
	    i=it.current();
	    break;
	}
	++it;
    }

    current++;
    lblInput->setText(tr("Input file: %1").arg(i->text(0)));
    QFileInfo fi(i->text(0));
    QString ext = "." + fi.extension();
    QString outputFile = i->text(0).replace(ext,".ogg");;
    if((radioDestination->isOn()) && (!editOutput->text().isEmpty()))
    {
	fi.setFile(outputFile);
	outputFile = editOutput->text()  + fi.fileName();
    }
    lblOutput->setText(tr("Output file: %1").arg(outputFile));

    log->append(tr("Running task %1 of %2").arg(current).arg(viewSource->childCount()));
    if(radioPro->isChecked())
	log->append(tr("Using built-in \"pro\" profile"));
    else if(radioPreview->isChecked())
	log->append(tr("Using built-in \"preview\" profile"));
    else
	log->append(tr("Using custom profile"));

	//get command arguments
	process = new QProcess(this);
	QStringList list = prepareArguments(i);
	

     for ( QStringList::Iterator stringIt = list.begin(); stringIt != list.end(); ++stringIt ) {
        process->addArgument(*stringIt);
    }

	connect(process,SIGNAL(readyReadStderr()),this,SLOT(updateLog()));
	connect(process,SIGNAL(processExited()),this,SLOT(completed()));
	//launch conversion
	process->start();
}


/*!
* \brief Starts execution of queue
*
* This slot is called when user clicks Start conversion button. If queue is not empty,
* current item index is set to zero and conversion of first video begins.
*/
void mainForm::start()
{
	log->clear();
	if(viewSource->childCount()==0)
	{
		log->append(tr("Task queue is empty!"));
		return;
	}
	current = 0;
	aborted = false;
	btnStart->setEnabled(false);
	btnStop->setEnabled(true);
	setFinishEnabled(pageConversion,false);
	//header = QString::null;
	log->append(tr("Starting ..."));
	startProcess();
}


/*!
* \brief Converts time from QString into long
*
* Converts time from QString in format hh:mm:ss into number of seconds.
* \param s time in format hh:mm:ss
* \return time in seconds for valid input, -1 otherwise
*/
long mainForm::stringToSecs(const QString &s)
{
    bool ok;
    QStringList list = QStringList::split(":",s);
      if(list.count()!=3)
	return -1;
    long hrs = list[0].toLong(&ok);
    if(!ok)
   	return -1;
    long mins = list[1].toLong(&ok);
    if(!ok)
	return -1;

    long secs = list[2].toLong(&ok);
    if(!ok)
	return -1;

      return hrs*3600+mins*60+secs;
}


/*!
* \brief Processes standard output of ffmpeg2theora process
*
* Reads, parses and displays standard output of \c ffmpeg2theora process. Tries to
* get and display video length, current position and remaining time.
*/
void mainForm::updateLog()
{
    QString text = (QString)(process->readStderr());
    text = text.simplifyWhiteSpace();

	//Video length has not yet been retrieved
    if(!parsed)
    {
	
	QStringList list = QStringList::split(".",text);
	if(list.count()>0 && list[0].right(8).contains(":")==2)
	{
	    lblLength->setText(tr("Length: %1").arg(list[0].right(8)));
	    duration = stringToSecs(list[0].right(8));
	    if(duration>0)
	    {
		progressCurrent->setTotalSteps(duration);		
	    }
	    parsed = true;
	}
    } 
    else
    {
	//Attempts to get position and remaining time	
	QString remaining = text.right(8);    
	QString position = "0"+text.left(7);
	int pos = stringToSecs(position);
	if(pos>0)
	{
	    QDateTime d(QDateTime::currentDateTime());
	    if(!timeStart)
		timeStart = d.toTime_t();
	    uint elapsed = d.toTime_t()-timeStart;
	    uint hr = (uint)(elapsed/3600);
	    uint min = (uint)((elapsed-3600*hr)/60);
	    uint sec = (uint)(elapsed - 60*min-hr*3600);
	    QTime elapsedTime(hr,min,sec);
	    lblElapsed->setText(tr("Elapsed time: %1").arg(elapsedTime.toString("hh:mm:ss")));
	    lblRemaining->setText(tr("Remaing time: %1").arg(remaining));
	    lblPosition->setText(tr("Position: %1").arg(position));
	    progressCurrent->setProgress(pos);
	    return;
	}
    }  	
    if(!text.contains("remaining"))  log->append(text); 
}


/*!
* \brief Notifies user about finished conversion
*
* Called when \c ffmpeg2theora process exits. If there are remaining unprocessed videos
* in queue, process is started again, otherwise application Finish button is again enabled.
*/
void mainForm::completed()
{
    timeStart = 0;
    if(process->normalExit())
    {
	log->append(tr("Conversion completed with success!"));
    } else
    {
	log->append(tr("Conversion failed!"));
    }
    delete process;
    process = 0;
    parsed = false;

    if((int)current<viewSource->childCount())
	startProcess();
    else
    {
	btnStart->setEnabled(true);
	btnStop->setEnabled(false);
	setFinishEnabled(pageConversion,true);
    }
}

/*!
* \brief Aborts conversion
*
* Allows user to kill \c ffmpeg2theora process
*/
void mainForm::killProcess()
{
	aborted = true;
	parsed = false;
	if(process)
	{	
	    current = viewSource->childCount();
		process->tryTerminate();
		QTimer::singleShot( 5000, process, SLOT( kill() ) );
	}
	log->append(tr("Aborted by user"));
}


/*!
* \brief Closes application if possible
*
* Closes application if no job is active, otherwise it displays dialog informing user about
* operation if progress.
*/
void mainForm::reject()
{
	if(btnStop->isEnabled()){
		QMessageBox::information(this,tr("ffmpeg2theora_fe"),tr("Conversion is in progress. Abort it if you wish to close this application"));
	}
	else
		QWizard::reject();
}

/*!
* \brief Removes current item
*
* Removes selected item from queue
*/
void mainForm::remove()
{
	QListViewItem *i = viewSource->currentItem();
	if(i)
	{
		viewSource->takeItem(i);
		delete i;
	}
	
}

/*!
* \brief Saves user preferences
*
* Shows dialog to save user preferences to profile file.
*/
void mainForm::saveConfig()
{
	QString fn = QFileDialog::getSaveFileName(
			QDir::homeDirPath(),  tr("Config files (*)"),
			this,  "save file dialog",  tr("Choose a filename to save under"));
	if(!fn.isEmpty())
	{
		QFile cfg_file(fn);
		if(cfg_file.exists())
			if(QMessageBox::question(this,tr("ffmpeg2theora_fe"),tr("Selected file already exists.\nDo you wish to overwrite it?"),tr("&Yes"),tr("&No")))
				return;
		
		
		if(!cfg_file.open(IO_WriteOnly)){
			QMessageBox::information( this, tr("ffmepg2theora_fe"),
																tr("Cannot save the file.\n"
																	 "Make sure you have writing rights."));
			return;
		}
		
		//Parse form fields
		QTextStream ts(&cfg_file);
		ts.setEncoding(QTextStream::UnicodeUTF8);
		ts << tr("# This file is genereted by ffmpeg2theora_fe. Do not edit manualy.") << endl;
		ts << tr("# Audio setup") << endl;
		ts << "audioquality=" << comboAudio->currentText() << endl;
		ts << "audiobitrate=" << editAudioRate->text() << endl;
		ts << "samplerate=" << editHz->text() << endl;
		ts << "channels=" << editChannels->text() << endl;
		if(checkSound->isOn())
			ts << "sound=off" << endl;
		else
			ts << "sound=on" <<endl;
		ts << "# Video options" << endl;
		ts << "videoquality=" << comboVideo->currentText() << endl;
		ts << "videobitrate=" << editVideoRate->text() << endl;
		ts << "width=" << editX->text() << endl;
		ts << "height=" << editY->text() << endl;
		ts << "inputfps=" << editFPS->text() << endl;
		ts << "outputfps=" << editOutputFPS->text() << endl;
		ts << "aspect=" << editAspect->text() << endl;
		ts << "croptop=" << editTop->text() << endl;
		ts << "cropbottom=" << editBottom->text() << endl;
		ts << "cropleft=" << editLeft->text() << endl;
		ts << "cropright=" << editRight->text() << endl;
		ts << "Image properties" << endl;
		ts << "sharpness=" << spinBoxSharpness->text() << endl;
		ts << "contrast=" << spinBoxContrast->text() << endl;
		ts << "brightness=" << spinBoxBrightness->text() << endl;
		ts << "gamma=" << spinBoxGamma->text() << endl;
		ts << "saturation=" << spinBoxSaturation->text() << endl;
		ts << "#[EOF]#" << endl;
	}
}


/*!
* \Brief Loads user preferences
*
* Displays dialog for loading user preferences from profile file.
*/
void mainForm::loadConfig()
{
	QString fn = QFileDialog::getOpenFileName(
			QDir::homeDirPath(),  tr("Config files (*)"),
			this,  "open file dialog",  tr("Select desired configuration file"));
	if(fn.isEmpty())
		return;
	QFile cfg_file(fn);
	if(!cfg_file.open(IO_ReadOnly))
	{
		QMessageBox::information( this, tr("ui4theora"),
															tr("Cannot open the file.\n"
																 "Permission denied."));
		return;
	}
	
	
	QTextStream ts(&cfg_file);
	ts.setEncoding(QTextStream::UnicodeUTF8);
	
	QString line, argument, value;
	int intValue;
	bool ok;
	
	//parse profile file
	while(!ts.atEnd())
	{
	    line = ts.readLine();
		line = line.simplifyWhiteSpace();
		line = line.lower();
		if(!(line.left(1)=="#")){
			
			
			argument = line.section('=',0,0);
			value = line.section('=',1,1);
			//Audio options
			if(argument == "audioquality")
				comboAudio->setCurrentText(value);
			else if(argument == "audiobitrate")
				editAudioRate->setText(value);
			else if(argument == "samplerate")
				editHz->setText(value);
			else if(argument == "channels")
				editChannels->setText(value);
			else if(argument == "sound")
			{
				if(value == "on")
					checkSound->setChecked(false);
				else
					checkSound->setChecked(true);
			}
			//Video options
			else if(argument == "videoquality")
				comboVideo->setCurrentText(value);
			else if(argument == "videobitrate")
				editVideoRate->setText(value);
			else if(argument == "width")
				editX->setText(value);
			else if(argument == "height")
				editY->setText(value);
			else if(argument == "inputfps")
				editFPS->setText(value);
			else if(argument == "outputfps")
				editOutputFPS->setText(value);
			else if(argument == "aspect")
			    editAspect->setText(value);
			//Crop
			else if(argument == "croptop")
				editTop->setText(value);
			else if(argument == "cropright")
				editRight->setText(value);
			else if(argument == "cropleft")
				editLeft->setText(value);
			else if(argument == "cropbottom")
				editBottom->setText(value);
			//image properties
			else if(argument == "sharpness")
			{
			    intValue = value.toInt(&ok);
			    spinBoxSharpness->setValue(intValue);
			}
			else if(argument == "contrast")
			{
			    intValue = value.toInt(&ok);
			    spinBoxContrast->setValue(intValue);
			}
			else if(argument == "brightness")
			{
			    intValue = value.toInt(&ok);
			    spinBoxBrightness->setValue(intValue);	
			}
			else if(argument == "gamma")
			{
			    intValue = value.toInt(&ok);
			    spinBoxGamma->setValue(intValue);
			}
			else if(argument == "saturation")
			{
			    intValue = value.toInt(&ok);
			    spinBoxSaturation->setValue(intValue);
			}
		    }
	}
}


/*!
* \brief Resets form fields
*
* Resets all form fields to their default values
*/
void mainForm::resetConfig()
{
	if(QMessageBox::question(this,tr("ffmpeg2theora_fe"),tr("Are you sure you wish to load default settings?"),tr("&Yes"),tr("&No")))
		return;
	comboAudio->setCurrentText("");
	checkSound->setChecked(false);
	editAudioRate->setText("");
	editHz->setText("");
	editChannels->setText("");
	
	comboVideo->setCurrentText("");
	editVideoRate->setText("");
	editFPS->setText("");
	editX->setText("");
	editY->setText("");
	editAspect->setText("");
	
	editTop->setText("");
	editBottom->setText("");
	editLeft->setText("");
	editRight->setText("");
	loadImageDefaults();
}


/*!
* \brief Shows ffmpeg2theora manual page
*
* Launches Konqueror to show \c ffmpeg2theora manual page.
*/
void mainForm::showHelp()
{
	if(help)
		return;
	help = new QProcess(this);	
	help->addArgument("konqueror");	
	help->addArgument("man:/ffmpeg2theora");
	connect(help,SIGNAL(processExited()),this,SLOT(helpClosed()));
	help->start();
}

/*!
* \brief Cleans Konqueror process variable
*
* Called when Konqueror is closed, this deletes \c help variable.
*/
void mainForm::helpClosed()
{
	delete help;
	help = 0;
}


/*!
* \brief Shows About QT dialog.
*
* Shows About QT dialog.
*/
void mainForm::aboutQT()
{
	QMessageBox::aboutQt(this,tr("About QT"));
}

/*!
* \brief Shows About dialog.
*
* Shows application about dialog.
*/
void mainForm::about()
{
	QMessageBox::about(this, tr("About ffmpeg2theora_fe"),tr("<h2>ffmpeg2theora_fe: Release 2</h2>"
								 
								"<p>ffmpeg2theora_fe is graphical user interface for <i>ffmpeg2theora</i></p>"
								"<p>created by Vladimir Jary</p>"
								"<p>E-mail: <a href=\"mailto:vljary.urk@centrum.cz\">vljary.urk@centrum.cz</a></p>"
								"<p>Web: <a href=\"http://kmlinux.fjfi.cvut.cz/~jaryv1\">http://kmlinux.fjfi.cvut.cz/~jaryv1</a></p>"
								"<p>This program may be used, modified and distributed under terms of  GPL</p>"));
}


/*!
* \brief Enables or disables several widget of wizard
*
* Enables or disable several widget of wizard. State depends on selected
* \c ffmpeg2theora profile. Widgets are disabled for built-in profiles (i. e. Pro and Preview)
* and enabled for custom presets
* \param state desired state of widgets
*/
void mainForm::togglePages( bool state )
{
    groupAudio->setEnabled(state);
    groupVideo->setEnabled(state);
    groupCrop->setEnabled(state);
    frameAdvancedPage->setEnabled(state);
    groupCustomPresets->setEnabled(state);
}

/*!
* \brief Resets image properties to default values
*
* Reset default values for sharpness, contrast, brightness, gamma correction
* and saturation
*/
void mainForm::loadImageDefaults()
{
    spinBoxSharpness->setValue(2);
    spinBoxContrast->setValue(10);
    spinBoxBrightness->setValue(0);
    spinBoxGamma->setValue(10);
    spinBoxSaturation->setValue(10);
}


/*!
* \brief Saves queue into shell script
*
* Allows user to save conversion queue into shell script.
*/
void mainForm::createScript()
{ 
    QString fn = QFileDialog::getSaveFileName(
	    QDir::homeDirPath(),  tr("Shell scripts (*.sh);;Any files(*)"),
	    this,  "save file dialog",  tr("Choose a filename to save under"));
    if(!fn.isEmpty())
    {
	QFile cfg_file(fn);
	if(cfg_file.exists())
	    if(QMessageBox::question(this,tr("ffmpeg2theora_fe"),tr("Selected script already exists.\nDo you wish to overwrite it?"),tr("&Yes"),tr("&No")))
		return;
	
	
	if(!cfg_file.open(IO_WriteOnly)){
	    QMessageBox::information( this, tr("ffmepg2theora_fe"),
				      tr("Cannot save the file.\n"
					 "Make sure you have writing rights."));
	    return;
	}
	
	QTextStream ts(&cfg_file);
	ts.setEncoding(QTextStream::UnicodeUTF8);
	ts << tr("#!/bin/sh") << endl;
	

	QStringList list;
	QString args;
	QString arg;
	QListViewItemIterator it( viewSource);
	while ( it.current() ) {
	    args = "";
		list = prepareArguments(it.current());
		for ( QStringList::Iterator stringIt = list.begin(); stringIt != list.end(); ++stringIt ) {
		    arg = *stringIt;
		    arg = arg.replace(QChar(' '),"\\ ");
		    args += arg + " ";
		}
		ts << args << endl << endl;
	    ++it;
	}
    }
}
